# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

from .MoneroRctKeyPublic import MoneroRctKeyPublic


class MoneroOutputEntry(p.MessageType):

    def __init__(
        self,
        idx: int = None,
        key: MoneroRctKeyPublic = None,
    ) -> None:
        self.idx = idx
        self.key = key

    @classmethod
    def get_fields(cls):
        return {
            1: ('idx', p.UVarintType, 0),
            2: ('key', MoneroRctKeyPublic, 0),
        }
