# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class PassphraseRequest(p.MessageType):
    MESSAGE_WIRE_TYPE = 41

    def __init__(
        self,
        on_device: bool = None,
    ) -> None:
        self.on_device = on_device

    @classmethod
    def get_fields(cls):
        return {
            1: ('on_device', p.BoolType, 0),
        }
