# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class PassphraseAck(p.MessageType):
    MESSAGE_WIRE_TYPE = 42

    def __init__(
        self,
        passphrase: str = None,
        state: bytes = None,
    ) -> None:
        self.passphrase = passphrase
        self.state = state

    @classmethod
    def get_fields(cls):
        return {
            1: ('passphrase', p.UnicodeType, 0),
            2: ('state', p.BytesType, 0),
        }
