# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

from .MoneroTransferDetails import MoneroTransferDetails

if __debug__:
    try:
        from typing import List
    except ImportError:
        List = None  # type: ignore


class MoneroKeyImageSyncStepRequest(p.MessageType):
    MESSAGE_WIRE_TYPE = 532

    def __init__(
        self,
        tdis: List[MoneroTransferDetails] = None,
    ) -> None:
        self.tdis = tdis if tdis is not None else []

    @classmethod
    def get_fields(cls):
        return {
            1: ('tdis', MoneroTransferDetails, p.FLAG_REPEATED),
        }
