# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class BinanceCancelMsg(p.MessageType):
    MESSAGE_WIRE_TYPE = 708

    def __init__(
        self,
        refid: str = None,
        sender: str = None,
        symbol: str = None,
    ) -> None:
        self.refid = refid
        self.sender = sender
        self.symbol = symbol

    @classmethod
    def get_fields(cls):
        return {
            1: ('refid', p.UnicodeType, 0),
            2: ('sender', p.UnicodeType, 0),
            3: ('symbol', p.UnicodeType, 0),
        }
