# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

from .EosPermissionLevel import EosPermissionLevel


class EosAuthorizationAccount(p.MessageType):

    def __init__(
        self,
        account: EosPermissionLevel = None,
        weight: int = None,
    ) -> None:
        self.account = account
        self.weight = weight

    @classmethod
    def get_fields(cls):
        return {
            1: ('account', EosPermissionLevel, 0),
            2: ('weight', p.UVarintType, 0),
        }
