# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class MoneroRctKeyPublic(p.MessageType):

    def __init__(
        self,
        dest: bytes = None,
        commitment: bytes = None,
    ) -> None:
        self.dest = dest
        self.commitment = commitment

    @classmethod
    def get_fields(cls):
        return {
            1: ('dest', p.BytesType, 0),
            2: ('commitment', p.BytesType, 0),
        }
