# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

from .MoneroTransactionSourceEntry import MoneroTransactionSourceEntry


class MoneroTransactionSetInputRequest(p.MessageType):
    MESSAGE_WIRE_TYPE = 503

    def __init__(
        self,
        src_entr: MoneroTransactionSourceEntry = None,
    ) -> None:
        self.src_entr = src_entr

    @classmethod
    def get_fields(cls):
        return {
            1: ('src_entr', MoneroTransactionSourceEntry, 0),
        }
