# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class DebugLinkDecision(p.MessageType):
    MESSAGE_WIRE_TYPE = 100

    def __init__(
        self,
        yes_no: bool = None,
        up_down: bool = None,
        input: str = None,
    ) -> None:
        self.yes_no = yes_no
        self.up_down = up_down
        self.input = input

    @classmethod
    def get_fields(cls):
        return {
            1: ('yes_no', p.BoolType, 0),
            2: ('up_down', p.BoolType, 0),
            3: ('input', p.UnicodeType, 0),
        }
