# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

if __debug__:
    try:
        from typing import List
    except ImportError:
        List = None  # type: ignore


class TezosProposalOp(p.MessageType):

    def __init__(
        self,
        source: bytes = None,
        period: int = None,
        proposals: List[bytes] = None,
    ) -> None:
        self.source = source
        self.period = period
        self.proposals = proposals if proposals is not None else []

    @classmethod
    def get_fields(cls):
        return {
            1: ('source', p.BytesType, 0),
            2: ('period', p.UVarintType, 0),
            4: ('proposals', p.BytesType, p.FLAG_REPEATED),
        }
