# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

from .MoneroTransactionRsigData import MoneroTransactionRsigData


class MoneroTransactionAllOutSetRequest(p.MessageType):
    MESSAGE_WIRE_TYPE = 513

    def __init__(
        self,
        rsig_data: MoneroTransactionRsigData = None,
    ) -> None:
        self.rsig_data = rsig_data

    @classmethod
    def get_fields(cls):
        return {
            1: ('rsig_data', MoneroTransactionRsigData, 0),
        }
