# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class MoneroLiveRefreshStepAck(p.MessageType):
    MESSAGE_WIRE_TYPE = 555

    def __init__(
        self,
        salt: bytes = None,
        key_image: bytes = None,
    ) -> None:
        self.salt = salt
        self.key_image = key_image

    @classmethod
    def get_fields(cls):
        return {
            1: ('salt', p.BytesType, 0),
            2: ('key_image', p.BytesType, 0),
        }
