# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class EosActionBuyRamBytes(p.MessageType):

    def __init__(
        self,
        payer: int = None,
        receiver: int = None,
        bytes: int = None,
    ) -> None:
        self.payer = payer
        self.receiver = receiver
        self.bytes = bytes

    @classmethod
    def get_fields(cls):
        return {
            1: ('payer', p.UVarintType, 0),
            2: ('receiver', p.UVarintType, 0),
            3: ('bytes', p.UVarintType, 0),
        }
