# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class EosActionUnknown(p.MessageType):

    def __init__(
        self,
        data_size: int = None,
        data_chunk: bytes = None,
    ) -> None:
        self.data_size = data_size
        self.data_chunk = data_chunk

    @classmethod
    def get_fields(cls):
        return {
            1: ('data_size', p.UVarintType, 0),
            2: ('data_chunk', p.BytesType, 0),
        }
