# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class ECDHSessionKey(p.MessageType):
    MESSAGE_WIRE_TYPE = 62

    def __init__(
        self,
        session_key: bytes = None,
    ) -> None:
        self.session_key = session_key

    @classmethod
    def get_fields(cls):
        return {
            1: ('session_key', p.BytesType, 0),
        }
